/*******************************************************************************
 * Copyhacked (H) 2012-2025.
 * This program and the accompanying materials
 * are made available under no term at all, use it like
 * you want, but share and discuss it
 * every time possible with every body.
 * 
 * Contributors:
 *      ron190 at ymail dot com - initial implementation
 ******************************************************************************/
package com.jsql.view.swing.manager;

import com.jsql.model.accessible.ExploitMethod;
import com.jsql.model.exception.JSqlException;
import com.jsql.util.I18nUtil;
import com.jsql.util.LogLevelUtil;
import com.jsql.view.swing.manager.util.*;
import com.jsql.view.swing.text.JPopupTextField;
import com.jsql.view.swing.text.JTextFieldPlaceholder;
import com.jsql.view.swing.text.JToolTipI18n;
import com.jsql.view.swing.util.I18nViewUtil;
import com.jsql.view.swing.util.MediatorHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicReference;

/**
 * Manager for uploading PHP SQL shell to the host and send queries.
 */
public class ManagerExploit extends AbstractManagerList {

    /**
     * Log4j logger sent to view.
     */
    private static final Logger LOGGER = LogManager.getRootLogger();

    private final AtomicReference<JTextField> username = new AtomicReference<>();
    private final AtomicReference<JTextField> password = new AtomicReference<>();
    private final AtomicReference<JTextField> netshare = new AtomicReference<>();
    protected final JTextField textfieldUrlShell;

    public static final String KEY_UDF_TAB = "UDF_TAB";
    public static final String KEY_WEBSHELL_TAB = "WEBSHELL_TAB";
    public static final String KEY_SQLSHELL_TAB = "SQLSHELL_TAB";
    public static final String KEY_UPLOAD_TAB = "UPLOAD_TAB";

    private final JComboBox<ModelItemType> comboBoxExploitTypes = new JComboBox<>(new ModelItemType[]{
        new ModelItemType(ManagerExploit.KEY_UDF_TAB, "UDF_TOOLTIP"),
        new ModelItemType(ManagerExploit.KEY_WEBSHELL_TAB, "WEBSHELL_TOOLTIP"),
        new ModelItemType(ManagerExploit.KEY_SQLSHELL_TAB, "SQLSHELL_TOOLTIP"),
        new ModelItemType(ManagerExploit.KEY_UPLOAD_TAB, "UPLOAD_TOOLTIP"),
    });

    private final JComboBox<Object> comboBoxExploitMethods = new JComboBox<>(new Object[]{
        ExploitMethod.AUTO,
        ComboBoxMethodRenderer.SEPARATOR,
        ExploitMethod.QUERY_BODY,
        ExploitMethod.TEMP_TABLE,
        ComboBoxMethodRenderer.SEPARATOR,
        ExploitMethod.NETSHARE
    });

    public ManagerExploit() {
        super("swing/list/payload.txt");

        var tooltipShellUrl = new AtomicReference<>(new JToolTipI18n(I18nUtil.valueByKey("SHELL_URL_TOOLTIP")));
        var placeholderResult = new JTextFieldPlaceholder(I18nUtil.valueByKey("SHELL_URL_LABEL")) {
            @Override
            public JToolTip createToolTip() {
                return tooltipShellUrl.get();
            }
        };
        this.textfieldUrlShell = new JPopupTextField(placeholderResult).getProxy();
        I18nViewUtil.addComponentForKey("SHELL_URL_LABEL", this.textfieldUrlShell);
        I18nViewUtil.addComponentForKey("SHELL_URL_TOOLTIP", tooltipShellUrl.get());
        this.textfieldUrlShell.setToolTipText(I18nUtil.valueByKey("SHELL_URL_TOOLTIP"));

        this.buildRunButton("SHELL_RUN_BUTTON_LABEL", "SHELL_RUN_BUTTON_TOOLTIP");
        this.run.setEnabled(false);
        this.buildPrivilege();

        var southPanel = new JPanel();
        southPanel.setLayout(new BoxLayout(southPanel, BoxLayout.Y_AXIS));
        southPanel.add(this.textfieldUrlShell);
        southPanel.add(this.lastLine);
        this.add(southPanel, BorderLayout.SOUTH);

        var userPassPanel = new JPanel();
        var groupLayout = new GroupLayout(userPassPanel);
        userPassPanel.setLayout(groupLayout);

        this.run.addActionListener(new ActionExploit(this.comboBoxExploitTypes));

        Arrays.asList(
            new ModelExploit(this.netshare, "EXPLOIT_NETSHARE_LABEL", "EXPLOIT_NETSHARE_TOOLTIP"),
            new ModelExploit(this.username, "SQL_SHELL_USERNAME_LABEL", "SQL_SHELL_USERNAME_TOOLTIP"),
            new ModelExploit(this.password, "SQL_SHELL_PASSWORD_LABEL", "SQL_SHELL_PASSWORD_TOOLTIP")
        ).forEach(model -> {
            var tooltip = new AtomicReference<>(new JToolTipI18n(I18nUtil.valueByKey(model.tooltipI18n)));
            model.textfield.set(new JPopupTextField(new JTextFieldPlaceholder(I18nUtil.valueByKey(model.labelI18n)) {
                @Override
                public JToolTip createToolTip() {
                    return tooltip.get();
                }
            }).getProxy());
            I18nViewUtil.addComponentForKey(model.labelI18n, model.textfield.get());
            I18nViewUtil.addComponentForKey(model.tooltipI18n, tooltip.get());
            model.textfield.get().setToolTipText(I18nUtil.valueByKey(model.tooltipI18n));
        });

        Arrays.asList(this.username.get(), this.password.get(), this.scrollListPaths, this.textfieldUrlShell, this.netshare.get())
        .forEach(component -> component.setVisible(false));

        this.comboBoxExploitTypes.setRenderer(new ComboBoxTypeRenderer());
        this.comboBoxExploitTypes.addItemListener(e -> {
            if (e.getStateChange() != ItemEvent.SELECTED) {
                return;
            }
            Arrays.asList(this.username.get(), this.password.get(), this.scrollListPaths, this.textfieldUrlShell)
            .forEach(component -> component.setVisible(false));
            ModelItemType selectedItem = (ModelItemType) e.getItem();
            if (!ManagerExploit.KEY_UDF_TAB.equals(selectedItem.getKeyLabel())) {
                this.scrollListPaths.setVisible(true);
                this.textfieldUrlShell.setVisible(true);
                if (ManagerExploit.KEY_SQLSHELL_TAB.equals(selectedItem.getKeyLabel())) {
                    this.username.get().setVisible(true);
                    this.password.get().setVisible(true);
                }
            }
            this.updateUI();  // required to adapt panel
        });
        this.comboBoxExploitMethods.addItemListener(e -> {
            if (e.getStateChange() == ItemEvent.SELECTED && e.getItem() instanceof ExploitMethod) {
                ExploitMethod selectedItem = (ExploitMethod) e.getItem();
                this.netshare.get().setVisible(false);
                if (selectedItem.equals(ExploitMethod.NETSHARE)) {
                    this.netshare.get().setVisible(true);
                }
                this.updateUI();  // required to adapt panel
            }
        });

        this.comboBoxExploitMethods.setRenderer(new ComboBoxMethodRenderer());
        this.comboBoxExploitMethods.addActionListener(new SeparatorListener(this.comboBoxExploitMethods));
        var labelUsing = new JLabel("using");
        groupLayout.setHorizontalGroup(
            groupLayout
            .createParallelGroup()
            .addGroup(
                groupLayout
                .createSequentialGroup()
                .addComponent(this.comboBoxExploitTypes)
                .addComponent(labelUsing, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE)
                .addComponent(this.comboBoxExploitMethods, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE, GroupLayout.PREFERRED_SIZE)
            )
            .addGroup(
                groupLayout.createParallelGroup()
                .addComponent(this.netshare.get())
                .addComponent(this.username.get())
                .addComponent(this.password.get())
            )
        );

        groupLayout.setVerticalGroup(
            groupLayout
            .createSequentialGroup()
            .addGroup(
                groupLayout
                .createParallelGroup(GroupLayout.Alignment.BASELINE)
                .addComponent(this.comboBoxExploitTypes)
                .addComponent(labelUsing)
                .addComponent(this.comboBoxExploitMethods)
            )
            .addGroup(
                groupLayout
                .createParallelGroup()
                .addComponent(this.netshare.get())
            )
            .addGroup(
                groupLayout
                .createParallelGroup()
                .addComponent(this.username.get())
            )
            .addGroup(
                groupLayout
                .createParallelGroup()
                .addComponent(this.password.get())
            )
        );
        
        this.add(userPassPanel, BorderLayout.NORTH);
    }

    protected class ActionExploit implements ActionListener {
        private final JComboBox<ModelItemType> comboBoxExploitTypes;

        public ActionExploit(JComboBox<ModelItemType> comboBoxExploitTypes) {
            this.comboBoxExploitTypes = comboBoxExploitTypes;
        }

        @Override
        public void actionPerformed(ActionEvent evt) {
            var modelSelectItem = (ModelItemType) this.comboBoxExploitTypes.getSelectedItem();
            var labelSelectItem = Objects.requireNonNull(modelSelectItem).getKeyLabel();
            if (ManagerExploit.KEY_UDF_TAB.equals(labelSelectItem)) {
                new SwingWorker<>() {
                    @Override
                    protected Object doInBackground() { Thread.currentThread().setName("SwingWorkerExploitUdf");
                        ActionExploit.this.start(null, null, null);
                        return null;
                    }
                }.doInBackground();
                return;
            }
            if (
                ManagerExploit.KEY_SQLSHELL_TAB.equals(labelSelectItem)
                && ManagerExploit.this.password.get().getText().isEmpty()
                || ManagerExploit.this.username.get().getText().isEmpty()
            ) {
                LOGGER.log(LogLevelUtil.CONSOLE_ERROR, "Missing credentials: manually search for file containing hardcoded credentials and read the file");
                return;
            }
            if (ManagerExploit.this.listPaths.getSelectedValuesList().isEmpty()) {
                LOGGER.log(LogLevelUtil.CONSOLE_ERROR, "Select at least one directory in the list");
                return;
            }
            String urlShell = ManagerExploit.this.textfieldUrlShell.getText();
            if (!urlShell.isEmpty() && !urlShell.matches("(?i)^https?://.*")) {
                if (!urlShell.matches("(?i)^\\w+://.*")) {
                    LOGGER.log(LogLevelUtil.CONSOLE_INFORM, "Undefined shell URL protocol, forcing to [https://]");
                    urlShell = "https://"+ urlShell;
                } else {
                    LOGGER.log(LogLevelUtil.CONSOLE_ERROR, "Unknown URL protocol");
                    return;
                }
            }
            if (StringUtils.isNotEmpty(urlShell)) {
                try {
                    new URI(urlShell);
                } catch (URISyntaxException e) {
                    LOGGER.log(LogLevelUtil.CONSOLE_ERROR, String.format("Incorrect URL: %s", e.getMessage()));
                    return;
                }
            }

            AtomicReference<File> fileToUpload = new AtomicReference<>();
            if (ManagerExploit.KEY_UPLOAD_TAB.equals(labelSelectItem)) {
                fileToUpload.set(ManagerExploit.chooseFile());
                if (fileToUpload.get() == null) {
                    return;
                }
            }

            String urlShellFinal = urlShell;
            new SwingWorker<>() {
                @Override
                protected Object doInBackground() { Thread.currentThread().setName("SwingWorkerExploitNonUdf");
                    ManagerExploit.this.horizontalGlue.setVisible(false);
                    ManagerExploit.this.progressBar.setVisible(true);
                    ManagerExploit.this.listPaths.getSelectedValuesList().forEach(remotePathFolder -> {
                        LOGGER.log(LogLevelUtil.CONSOLE_DEFAULT, String.format("Checking path [%s]...", remotePathFolder));
                        ActionExploit.this.start(remotePathFolder.toString(), urlShellFinal, fileToUpload.get());
                    });
                    ManagerExploit.this.endProcess();
                    return null;
                }
            }.doInBackground();
        }

        private void start(String remotePathFolder, String urlShellFinal, File fileToUpload) {
            try {
                ManagerExploit.this.createPayload(remotePathFolder, urlShellFinal, fileToUpload);
            } catch (JSqlException e) {
                LOGGER.log(LogLevelUtil.CONSOLE_ERROR, String.format("Payload creation failure: %s", e.getMessage()));
            }
        }
    }

    private static File chooseFile() {
        var filechooser = new JFileChooser(MediatorHelper.model().getMediatorUtils().getPreferencesUtil().getPathFile());
        filechooser.setDialogTitle(I18nUtil.valueByKey("UPLOAD_DIALOG_TEXT"));
        int returnVal = filechooser.showOpenDialog(MediatorHelper.frame());
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            return filechooser.getSelectedFile();
        }
        return null;
    }

    protected void createPayload(String remotePathFolder, String urlShell, File fileToUpload) throws JSqlException {
        var exploitMethod = ExploitMethod.forName(Objects.requireNonNull(this.comboBoxExploitMethods.getSelectedItem()).toString())
        .orElse(ExploitMethod.AUTO);

        if (remotePathFolder != null && !remotePathFolder.endsWith("/")) {
            remotePathFolder += "/";
        }
        String pathNetshare = null;
        if (exploitMethod == ExploitMethod.NETSHARE && !this.netshare.get().getText().endsWith("\\")) {
            pathNetshare = String.format("%s%s", this.netshare.get().getText(), "\\");
        }

        var modelItemType = (ModelItemType) this.comboBoxExploitTypes.getSelectedItem();
        if (ManagerExploit.KEY_UDF_TAB.equals(Objects.requireNonNull(modelItemType).getKeyLabel())) {
            MediatorHelper.model().getUdfAccess().createUdf(this.netshare.get().getText(), exploitMethod);
        } else if (ManagerExploit.KEY_WEBSHELL_TAB.equals(modelItemType.getKeyLabel())) {
            MediatorHelper.model().getResourceAccess().createExploitWeb(
                remotePathFolder,
                urlShell,
                pathNetshare,
                exploitMethod
            );
        } else if (ManagerExploit.KEY_SQLSHELL_TAB.equals(modelItemType.getKeyLabel())) {
            MediatorHelper.model().getResourceAccess().createExploitSql(
                remotePathFolder,
                urlShell,
                pathNetshare,
                exploitMethod,
                this.username.get().getText(),
                this.password.get().getText()
            );
        } else if (ManagerExploit.KEY_UPLOAD_TAB.equals(modelItemType.getKeyLabel())) {
            MediatorHelper.model().getResourceAccess().createExploitUpload(
                remotePathFolder,
                urlShell,
                pathNetshare,
                exploitMethod,
                fileToUpload
            );
        }
    }
}
